"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._generateCode = exports.ControllerPlugin = void 0;
const crypto_1 = __importDefault(require("crypto"));
const express_1 = __importDefault(require("express"));
const util_1 = __importDefault(require("util"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const lib = __importStar(require("@clusterio/lib"));
const controller_1 = require("@clusterio/controller");
const { basicType } = lib;
const messages_1 = require("./messages");
async function generateCode(length) {
    // ji1lI, 0oOQ, and 2Z are not present to ease reading.
    let letters = "abcdefghkmnpqrstuvwxyzABCDEFGHJKLMNPRSTUVWXY3456789";
    let asyncRandomBytes = util_1.default.promisify(crypto_1.default.randomBytes);
    let code = [];
    for (let byte of await asyncRandomBytes(length)) {
        // Due to the 51 characters in the letters not fitting perfectly in
        // 256 there's an ever so slight bias towards a with this algorithm.
        code.push(letters[byte % letters.length]);
    }
    return code.join("");
}
class ControllerPlugin extends controller_1.BaseControllerPlugin {
    players;
    async init() {
        // Store of validation attempts by players
        this.players = new Map();
        // Periodically remove expired entries
        setInterval(() => {
            let now = Date.now();
            for (let [player, entry] of this.players) {
                if (entry.expiresMs < now) {
                    this.players.delete(player);
                }
            }
        }, 60e3).unref();
        this.controller.app.get("/api/player_auth/servers", (req, res) => {
            let servers = [];
            for (let instance of this.controller.instances.values()) {
                if (instance.status === "running" && instance.config.get("player_auth.load_plugin")) {
                    servers.push(instance.config.get("factorio.settings")["name"] || "unnamed server");
                }
            }
            res.send(servers);
        });
        this.controller.app.post("/api/player_auth/player_code", express_1.default.json(), (req, res, next) => {
            this.handlePlayerCode(req, res).catch(next);
        });
        this.controller.app.post("/api/player_auth/verify", express_1.default.json(), (req, res, next) => {
            this.handleVerify(req, res).catch(next);
        });
        this.controller.handle(messages_1.FetchPlayerCodeRequest, this.handleFetchPlayerCodeRequest.bind(this));
        this.controller.handle(messages_1.SetVerifyCodeRequest, this.handleSetVerifyCodeRequest.bind(this));
    }
    async handlePlayerCode(req, res) {
        if (basicType(req.body) !== "object") {
            res.sendStatus(400);
            return;
        }
        let playerCode = req.body.player_code;
        if (typeof playerCode !== "string") {
            res.sendStatus(400);
            return;
        }
        for (let entry of this.players.values()) {
            if (entry.playerCode === playerCode && entry.expiresMs > Date.now()) {
                let verifyCode = await generateCode(this.controller.config.get("player_auth.code_length"));
                let secret = Buffer.from(this.controller.config.get("controller.auth_secret"), "base64");
                let verifyToken = jsonwebtoken_1.default.sign({
                    aud: "player_auth.verify_code",
                    exp: Math.floor(entry.expiresMs / 1000),
                    verify_code: verifyCode,
                    player_code: playerCode,
                }, secret);
                res.send({ verify_code: verifyCode, verify_token: verifyToken });
                return;
            }
        }
        res.send({ error: true, message: "invalid player_code" });
    }
    async handleVerify(req, res) {
        if (basicType(req.body) !== "object") {
            res.sendStatus(400);
            return;
        }
        let playerCode = req.body.player_code;
        if (typeof playerCode !== "string") {
            res.sendStatus(400);
            return;
        }
        let verifyCode = req.body.verify_code;
        if (typeof verifyCode !== "string") {
            res.sendStatus(400);
            return;
        }
        let verifyToken = req.body.verify_token;
        if (typeof verifyToken !== "string") {
            res.sendStatus(400);
            return;
        }
        let secret = Buffer.from(this.controller.config.get("controller.auth_secret"), "base64");
        try {
            let payload = jsonwebtoken_1.default.verify(verifyToken, secret, { audience: "player_auth.verify_code" });
            if (payload.verify_code !== verifyCode) {
                throw new Error("invalid verify_code");
            }
            if (payload.player_code !== playerCode) {
                throw new Error("invalid player_code");
            }
        }
        catch (err) {
            res.send({ error: true, message: err.message });
            return;
        }
        for (let [player, entry] of this.players) {
            if (entry.playerCode === playerCode && entry.expiresMs > Date.now()) {
                if (entry.verifyCode === verifyCode) {
                    let user = this.controller.userManager.getByName(player);
                    if (!user) {
                        res.send({ error: true, message: "invalid user" });
                        return;
                    }
                    let token = this.controller.userManager.signUserToken(user);
                    res.send({ verified: true, token });
                    return;
                }
                res.send({ verified: false });
                return;
            }
        }
        res.send({ error: true, message: "invalid player_code" });
    }
    async handleFetchPlayerCodeRequest(request) {
        let playerCode = await generateCode(this.controller.config.get("player_auth.code_length"));
        let expiresMs = Date.now() + this.controller.config.get("player_auth.code_timeout") * 1000;
        this.players.set(request.player, { playerCode, verifyCode: null, expiresMs });
        return { playerCode, controllerUrl: this.controller.getControllerUrl() };
    }
    async handleSetVerifyCodeRequest(request) {
        let { player, verifyCode } = request;
        let entry = this.players.get(player);
        if (!entry || entry.expiresMs < Date.now()) {
            throw new lib.RequestError("invalid player");
        }
        entry.verifyCode = verifyCode;
    }
}
exports.ControllerPlugin = ControllerPlugin;
// For testing only
exports._generateCode = generateCode;
//# sourceMappingURL=controller.js.map