"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InstancePlugin = void 0;
const host_1 = require("@clusterio/host");
const messages_1 = require("./messages");
class InstancePlugin extends host_1.BaseInstancePlugin {
    async init() {
        if (!this.instance.config.get("factorio.enable_save_patching")) {
            throw new Error("player_auth plugin requires save patching.");
        }
        this.instance.server.on("ipc-player_auth", (ipcPlayerAuth) => this.handleEvent(ipcPlayerAuth).catch(err => this.logger.error(`Error handling event:\n${err.stack}`)));
    }
    async handleEvent(event) {
        if (event.type === "open_dialog") {
            if (!this.host.connector.connected) {
                await this.sendRcon(`/web-login error ${event.player} login is temporarily unavailable`);
                return;
            }
            let response;
            try {
                response = await this.instance.sendTo("controller", new messages_1.FetchPlayerCodeRequest(event.player));
            }
            catch (err) {
                await this.sendRcon(`/web-login error ${event.player} ${err.message}`);
                return;
            }
            await this.sendRcon(`/web-login open ${event.player} ${response.controllerUrl} ${response.playerCode}`);
        }
        else if (event.type === "set_verify_code") {
            try {
                await this.instance.sendTo("controller", new messages_1.SetVerifyCodeRequest(event.player, event.verify_code));
            }
            catch (err) {
                await this.sendRcon(`/web-login error ${event.player} ${err.message}`);
                return;
            }
            await this.sendRcon(`/web-login code_set ${event.player}`);
        }
    }
}
exports.InstancePlugin = InstancePlugin;
//# sourceMappingURL=instance.js.map